/* ----- Ctrl Port defines ----- */
#define		lcd_port	LATB
#define		lcd_e		LATAbits.LATA4
#define		lcd_rs		LATAbits.LATA5
/*     connection
  PIC    B7,B7,B5,B4.    A4,A5
  LCD    D7,D6,D5,D4,     E,RS,    RW==Lo
*/
#pragma code
/*-------------------------- Out Hi nibble -----------*/
void  k_Lcd_OutH ( Byte  v )
{
   lcd_port &= 0x0F;	     //  (|[gʕۑ)
   lcd_port |= ( v & 0xF0 ); //  f[^
   lcd_rs = 0;               //  RS  0:command
   lcd_e  = 1;               //  strobe
   Nop(); Nop();
   lcd_e  = 0;
   Delay_us(100);
}

/*-------------------------- Out Command Byte ----------*/
void  k_Lcd_OutC ( Byte  c ) {
   lcd_port &= 0x0F;	     //  (|[gʕۑ)
   lcd_port |= ( c & 0xF0 ); //  f[^
   lcd_rs = 0;               //  RS  0:command
   lcd_e  = 1;               //  strobe
   Nop(); Nop();
   lcd_e  = 0;
   lcd_port &= 0x0F;	     //  (|[gʕۑ)
   lcd_port |= ( (c<<4) & 0xF0 ); // f[^
   lcd_rs = 0;              //  RS  0:command
   lcd_e  = 1;              //  strobe
   Nop(); Nop();
   lcd_e  = 0;
   Delay_us(120);
}

/*-------------------------- Out Data Byte -------------*/
void  k_Lcd_OutD ( Byte  d ) {
   lcd_port &= 0x0F;	     // (|[gʕۑ)
   lcd_port |= ( d & 0xF0 ); // f[^
   lcd_rs = 1;               //  RS  1:data
   lcd_e  = 1;               //  strobe
   Nop(); Nop();
   lcd_e  = 0;
   lcd_port &= 0x0F;	     // (|[gʕۑ)
   lcd_port |= ( (d<<4) & 0xF0 ); // f[^
   lcd_rs = 1;              //  RS  1:data
   lcd_e  = 1;              //  strobe
   Nop(); Nop();
   lcd_e  = 0;
   Delay_us(120);
}

/*-------------------------- Initialize ----------------*/
void  k_Lcd_Init ( void ) {

   lcd_port = 0;

   Delay_ms(16);
   k_Lcd_OutH(0x30);
   Delay_ms(5);
   k_Lcd_OutH(0x30);
   k_Lcd_OutH(0x30);

   k_Lcd_OutH(0x20);  /*  4-bit mode  */
   k_Lcd_OutC(0x28);  /*  2-line, 5x7dot  */
   k_Lcd_OutC(0x06);  /*  cur move right ( -> ) */
}

/*-------------------------- Cls -----------------------*/
void  k_Lcd_Cls ( void ) {
   k_Lcd_OutC(0x01);
   Delay_ms(2);
}

/*-------------------------- Cur on/off -----------------*/
void  k_Lcd_Cur ( Byte  onoff ) {
   if ( onoff )           /*  1:on   */
      k_Lcd_OutC(0x0F);   /*     with blink  */
   else                   /*  0:off  */
      k_Lcd_OutC(0x0C);
}

/*-------------------------- Locate ----------------------*/
void  k_Lcd_Locate ( Byte  y, Byte x ) {
   x--;  x |=0x80;
   if (y == 2)   x |= 0x40;
   k_Lcd_OutC(x);
}

/*-------------------------- Print Str (RAM)--------------*/
void  k_Lcd_Print ( Byte *s ) {
   while ( *s != 0 ) {
      k_Lcd_OutD ( *s );
      s++;
   }
}
/*-------------------------- Print Str (ROM)--------------*/
void  k_Lcd_Print_ROM ( rom const Byte *s ) {
   while ( *s != 0 ) {
      k_Lcd_OutD ( *s );
      s++;
   }
}
